$(document).on 'ready page:load', ->
  $('table.editable_rows').each ->
    initEditableRows(this)
    initCreation(this)

initEditableRows = (table) ->
  $table = $(table)
  $table.find('a.edit').click (event) ->
    event.preventDefault()
    initRowEditing($(this))
  $table.find('a.destroy').click (event) ->
    event.preventDefault()
    event.stopPropagation()
    $button = $(this)
    confirmation = $button.data('confirm')
    if !confirmation or confirm(confirmation)
      $row = $button.closest('tr')
      $.ajax
        type: $button.data('method')
        url: $button.attr('href') + '.json'
        success: () -> $row.remove()
        error: (data) -> setErrors($row, data)

  initRowEditing = ($editButton) ->
    $controlsCell = $editButton.parent('td')
    $row = $controlsCell.parent('tr')
    orygControlHtml = $controlsCell.find('a').detach()
    initControls($controlsCell)
    initInputs($row)

    $row.find('td[data] input').keypress (event) ->
      if event.which == 13
        event.preventDefault()
        save()

    $controlsCell.find('.save').click (event) ->
      event.preventDefault()
      save()

    save = () ->
      clearErrors($row)
      data = {}
      $row.find('td[data]').each ->
        $cell = $(this)
        data[$cell.attr('data')] = $cell.find('input').val()
      ajaxData = {}
      ajaxData[$table.data('resource')] = data
      $.ajax
        type: "PUT"
        url: window.location.href + "/#{$row.data('id')}.json"
        data: ajaxData
        success: () -> restoreRow(true)
        error: (data) -> setErrors($row, data)

    restoreRow = (updated) ->
      $controlsCell.html(orygControlHtml)
      $row.find('td[data]').each ->
        $cell = $(this)
        $input = $cell.find('input')
        $cell.html(if updated then $input.val() else $input.data('undo'))

    $controlsCell.find('.cancel').click (event) ->
      event.preventDefault()
      restoreRow(false)

initCreation = (table) ->
  $table = $(table)
  $createButton = $table.find('a.create')
  $row = $createButton.closest('tr')
  initInputs($row)

  $row.find('td[data]').keypress (event) ->
    if event.which == 13
      event.preventDefault()
      create()

  $createButton.click (event) ->
    event.preventDefault()
    create()

  create = () ->
    clearErrors($row)
    data = {}
    $row.find('td[data]').each ->
      $cell = $(this)
      data[$cell.attr('data')] = $cell.find('input').val()
    ajaxData = {}
    ajaxData[$table.data('resource')] = data
    $.ajax
      type: "POST"
      url: window.location.href
      data: ajaxData
      success: (data) ->
        newTableBody = $("<div>#{data}</div>").find('table.editable_rows tbody').html()
        $table.find('tbody').html(newTableBody)
        $row.find('input:text').val('')
        initEditableRows(table)
      error: (data) -> setErrors($row, data)

initInputs = ($row) ->
  $row.find('td[data]').each ->
    $cell = $(this)
    value = $cell.html()
    $cell.html("<input data-undo='#{value}' value='#{value}'><span class='has-error'/>")

initControls = ($cell) ->
  $cell.html("<a class='btn btn-primary btn-xs save' href='#'><i class='fa fa-check'/></a> <a class='btn btn-default btn-xs cancel' href='#'><i class='fa fa-remove'/></a>")

clearErrors = ($row) ->
  $row.find("td[data] span").html('')

setErrors = ($row, data) ->
  for k,v of data.responseJSON
    $row.find("td[data='#{k}'] span").html(v.join())
